"""Request/response logging for Airbyte SDK."""

from .logger import NullLogger, RequestLogger
from .types import LogSession, RequestLog

__all__ = [
    "RequestLogger",
    "NullLogger",
    "RequestLog",
    "LogSession",
]



"""
Type definitions for amplitude connector.
"""
from __future__ import annotations

from ._vendored.connector_sdk.types import AirbyteHostedAuthConfig as AirbyteAuthConfig  # noqa: F401

# Use typing_extensions.TypedDict for Pydantic compatibility
try:
    from typing_extensions import TypedDict, NotRequired
except ImportError:
    from typing import TypedDict, NotRequired  # type: ignore[attr-defined]

from typing import Any, Literal


# ===== NESTED PARAM TYPE DEFINITIONS =====
# Nested parameter schemas discovered during parameter extraction

# ===== OPERATION PARAMS TYPE DEFINITIONS =====

class AnnotationsListParams(TypedDict):
    """Parameters for annotations.list operation"""
    pass

class AnnotationsGetParams(TypedDict):
    """Parameters for annotations.get operation"""
    annotation_id: str

class CohortsListParams(TypedDict):
    """Parameters for cohorts.list operation"""
    pass

class CohortsGetParams(TypedDict):
    """Parameters for cohorts.get operation"""
    cohort_id: str

class EventsListListParams(TypedDict):
    """Parameters for events_list.list operation"""
    pass

class ActiveUsersListParams(TypedDict):
    """Parameters for active_users.list operation"""
    start: str
    end: str
    m: NotRequired[str]
    i: NotRequired[int]
    g: NotRequired[str]

class AverageSessionLengthListParams(TypedDict):
    """Parameters for average_session_length.list operation"""
    start: str
    end: str

# ===== SEARCH TYPES =====

# Sort specification
AirbyteSortOrder = Literal["asc", "desc"]

# ===== ANNOTATIONS SEARCH TYPES =====

class AnnotationsSearchFilter(TypedDict, total=False):
    """Available fields for filtering annotations search queries."""
    date: str | None
    """The date when the annotation was made"""
    details: str | None
    """Additional details or information related to the annotation"""
    id: int | None
    """The unique identifier for the annotation"""
    label: str | None
    """The label assigned to the annotation"""


class AnnotationsInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    date: list[str]
    """The date when the annotation was made"""
    details: list[str]
    """Additional details or information related to the annotation"""
    id: list[int]
    """The unique identifier for the annotation"""
    label: list[str]
    """The label assigned to the annotation"""


class AnnotationsAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    date: Any
    """The date when the annotation was made"""
    details: Any
    """Additional details or information related to the annotation"""
    id: Any
    """The unique identifier for the annotation"""
    label: Any
    """The label assigned to the annotation"""


class AnnotationsStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    date: str
    """The date when the annotation was made"""
    details: str
    """Additional details or information related to the annotation"""
    id: str
    """The unique identifier for the annotation"""
    label: str
    """The label assigned to the annotation"""


class AnnotationsSortFilter(TypedDict, total=False):
    """Available fields for sorting annotations search results."""
    date: AirbyteSortOrder
    """The date when the annotation was made"""
    details: AirbyteSortOrder
    """Additional details or information related to the annotation"""
    id: AirbyteSortOrder
    """The unique identifier for the annotation"""
    label: AirbyteSortOrder
    """The label assigned to the annotation"""


# Entity-specific condition types for annotations
class AnnotationsEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: AnnotationsSearchFilter


class AnnotationsNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: AnnotationsSearchFilter


class AnnotationsGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: AnnotationsSearchFilter


class AnnotationsGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: AnnotationsSearchFilter


class AnnotationsLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: AnnotationsSearchFilter


class AnnotationsLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: AnnotationsSearchFilter


class AnnotationsLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: AnnotationsStringFilter


class AnnotationsFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: AnnotationsStringFilter


class AnnotationsKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: AnnotationsStringFilter


class AnnotationsContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: AnnotationsAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
AnnotationsInCondition = TypedDict("AnnotationsInCondition", {"in": AnnotationsInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

AnnotationsNotCondition = TypedDict("AnnotationsNotCondition", {"not": "AnnotationsCondition"}, total=False)
"""Negates the nested condition."""

AnnotationsAndCondition = TypedDict("AnnotationsAndCondition", {"and": "list[AnnotationsCondition]"}, total=False)
"""True if all nested conditions are true."""

AnnotationsOrCondition = TypedDict("AnnotationsOrCondition", {"or": "list[AnnotationsCondition]"}, total=False)
"""True if any nested condition is true."""

AnnotationsAnyCondition = TypedDict("AnnotationsAnyCondition", {"any": AnnotationsAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all annotations condition types
AnnotationsCondition = (
    AnnotationsEqCondition
    | AnnotationsNeqCondition
    | AnnotationsGtCondition
    | AnnotationsGteCondition
    | AnnotationsLtCondition
    | AnnotationsLteCondition
    | AnnotationsInCondition
    | AnnotationsLikeCondition
    | AnnotationsFuzzyCondition
    | AnnotationsKeywordCondition
    | AnnotationsContainsCondition
    | AnnotationsNotCondition
    | AnnotationsAndCondition
    | AnnotationsOrCondition
    | AnnotationsAnyCondition
)


class AnnotationsSearchQuery(TypedDict, total=False):
    """Search query for annotations entity."""
    filter: AnnotationsCondition
    sort: list[AnnotationsSortFilter]


# ===== COHORTS SEARCH TYPES =====

class CohortsSearchFilter(TypedDict, total=False):
    """Available fields for filtering cohorts search queries."""
    app_id: int | None
    """The unique identifier of the application"""
    archived: bool | None
    """Indicates if the cohort data is archived"""
    chart_id: str | None
    """The identifier of the chart associated with the cohort"""
    created_at: int | None
    """The timestamp when the cohort was created"""
    definition: dict[str, Any] | None
    """The specific definition or criteria for the cohort"""
    description: str | None
    """A brief explanation or summary of the cohort"""
    edit_id: str | None
    """The ID for editing purposes or version control"""
    finished: bool | None
    """Indicates if the cohort data has been finalized"""
    hidden: bool | None
    """Flag to determine if the cohort is hidden from view"""
    id: str | None
    """The unique identifier for the cohort"""
    is_official_content: bool | None
    """Indicates if the cohort data is official content"""
    is_predictive: bool | None
    """Flag to indicate if the cohort is predictive"""
    last_computed: int | None
    """Timestamp of the last computation of cohort data"""
    last_mod: int | None
    """Timestamp of the last modification made to the cohort"""
    last_viewed: int | None
    """Timestamp when the cohort was last viewed"""
    location_id: str | None
    """Identifier of the location associated with the cohort"""
    metadata: list[Any] | None
    """Additional information or data related to the cohort"""
    name: str | None
    """The name or title of the cohort"""
    owners: list[Any] | None
    """The owners or administrators of the cohort"""
    popularity: int | None
    """Popularity rank or score of the cohort"""
    published: bool | None
    """Status indicating if the cohort data is published"""
    shortcut_ids: list[Any] | None
    """Identifiers of any shortcuts associated with the cohort"""
    size: int | None
    """Size or scale of the cohort data"""
    type: str | None
    """The type or category of the cohort"""
    view_count: int | None
    """The total count of views on the cohort data"""
    viewers: list[Any] | None
    """Users or viewers who have access to the cohort data"""


class CohortsInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    app_id: list[int]
    """The unique identifier of the application"""
    archived: list[bool]
    """Indicates if the cohort data is archived"""
    chart_id: list[str]
    """The identifier of the chart associated with the cohort"""
    created_at: list[int]
    """The timestamp when the cohort was created"""
    definition: list[dict[str, Any]]
    """The specific definition or criteria for the cohort"""
    description: list[str]
    """A brief explanation or summary of the cohort"""
    edit_id: list[str]
    """The ID for editing purposes or version control"""
    finished: list[bool]
    """Indicates if the cohort data has been finalized"""
    hidden: list[bool]
    """Flag to determine if the cohort is hidden from view"""
    id: list[str]
    """The unique identifier for the cohort"""
    is_official_content: list[bool]
    """Indicates if the cohort data is official content"""
    is_predictive: list[bool]
    """Flag to indicate if the cohort is predictive"""
    last_computed: list[int]
    """Timestamp of the last computation of cohort data"""
    last_mod: list[int]
    """Timestamp of the last modification made to the cohort"""
    last_viewed: list[int]
    """Timestamp when the cohort was last viewed"""
    location_id: list[str]
    """Identifier of the location associated with the cohort"""
    metadata: list[list[Any]]
    """Additional information or data related to the cohort"""
    name: list[str]
    """The name or title of the cohort"""
    owners: list[list[Any]]
    """The owners or administrators of the cohort"""
    popularity: list[int]
    """Popularity rank or score of the cohort"""
    published: list[bool]
    """Status indicating if the cohort data is published"""
    shortcut_ids: list[list[Any]]
    """Identifiers of any shortcuts associated with the cohort"""
    size: list[int]
    """Size or scale of the cohort data"""
    type: list[str]
    """The type or category of the cohort"""
    view_count: list[int]
    """The total count of views on the cohort data"""
    viewers: list[list[Any]]
    """Users or viewers who have access to the cohort data"""


class CohortsAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    app_id: Any
    """The unique identifier of the application"""
    archived: Any
    """Indicates if the cohort data is archived"""
    chart_id: Any
    """The identifier of the chart associated with the cohort"""
    created_at: Any
    """The timestamp when the cohort was created"""
    definition: Any
    """The specific definition or criteria for the cohort"""
    description: Any
    """A brief explanation or summary of the cohort"""
    edit_id: Any
    """The ID for editing purposes or version control"""
    finished: Any
    """Indicates if the cohort data has been finalized"""
    hidden: Any
    """Flag to determine if the cohort is hidden from view"""
    id: Any
    """The unique identifier for the cohort"""
    is_official_content: Any
    """Indicates if the cohort data is official content"""
    is_predictive: Any
    """Flag to indicate if the cohort is predictive"""
    last_computed: Any
    """Timestamp of the last computation of cohort data"""
    last_mod: Any
    """Timestamp of the last modification made to the cohort"""
    last_viewed: Any
    """Timestamp when the cohort was last viewed"""
    location_id: Any
    """Identifier of the location associated with the cohort"""
    metadata: Any
    """Additional information or data related to the cohort"""
    name: Any
    """The name or title of the cohort"""
    owners: Any
    """The owners or administrators of the cohort"""
    popularity: Any
    """Popularity rank or score of the cohort"""
    published: Any
    """Status indicating if the cohort data is published"""
    shortcut_ids: Any
    """Identifiers of any shortcuts associated with the cohort"""
    size: Any
    """Size or scale of the cohort data"""
    type: Any
    """The type or category of the cohort"""
    view_count: Any
    """The total count of views on the cohort data"""
    viewers: Any
    """Users or viewers who have access to the cohort data"""


class CohortsStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    app_id: str
    """The unique identifier of the application"""
    archived: str
    """Indicates if the cohort data is archived"""
    chart_id: str
    """The identifier of the chart associated with the cohort"""
    created_at: str
    """The timestamp when the cohort was created"""
    definition: str
    """The specific definition or criteria for the cohort"""
    description: str
    """A brief explanation or summary of the cohort"""
    edit_id: str
    """The ID for editing purposes or version control"""
    finished: str
    """Indicates if the cohort data has been finalized"""
    hidden: str
    """Flag to determine if the cohort is hidden from view"""
    id: str
    """The unique identifier for the cohort"""
    is_official_content: str
    """Indicates if the cohort data is official content"""
    is_predictive: str
    """Flag to indicate if the cohort is predictive"""
    last_computed: str
    """Timestamp of the last computation of cohort data"""
    last_mod: str
    """Timestamp of the last modification made to the cohort"""
    last_viewed: str
    """Timestamp when the cohort was last viewed"""
    location_id: str
    """Identifier of the location associated with the cohort"""
    metadata: str
    """Additional information or data related to the cohort"""
    name: str
    """The name or title of the cohort"""
    owners: str
    """The owners or administrators of the cohort"""
    popularity: str
    """Popularity rank or score of the cohort"""
    published: str
    """Status indicating if the cohort data is published"""
    shortcut_ids: str
    """Identifiers of any shortcuts associated with the cohort"""
    size: str
    """Size or scale of the cohort data"""
    type: str
    """The type or category of the cohort"""
    view_count: str
    """The total count of views on the cohort data"""
    viewers: str
    """Users or viewers who have access to the cohort data"""


class CohortsSortFilter(TypedDict, total=False):
    """Available fields for sorting cohorts search results."""
    app_id: AirbyteSortOrder
    """The unique identifier of the application"""
    archived: AirbyteSortOrder
    """Indicates if the cohort data is archived"""
    chart_id: AirbyteSortOrder
    """The identifier of the chart associated with the cohort"""
    created_at: AirbyteSortOrder
    """The timestamp when the cohort was created"""
    definition: AirbyteSortOrder
    """The specific definition or criteria for the cohort"""
    description: AirbyteSortOrder
    """A brief explanation or summary of the cohort"""
    edit_id: AirbyteSortOrder
    """The ID for editing purposes or version control"""
    finished: AirbyteSortOrder
    """Indicates if the cohort data has been finalized"""
    hidden: AirbyteSortOrder
    """Flag to determine if the cohort is hidden from view"""
    id: AirbyteSortOrder
    """The unique identifier for the cohort"""
    is_official_content: AirbyteSortOrder
    """Indicates if the cohort data is official content"""
    is_predictive: AirbyteSortOrder
    """Flag to indicate if the cohort is predictive"""
    last_computed: AirbyteSortOrder
    """Timestamp of the last computation of cohort data"""
    last_mod: AirbyteSortOrder
    """Timestamp of the last modification made to the cohort"""
    last_viewed: AirbyteSortOrder
    """Timestamp when the cohort was last viewed"""
    location_id: AirbyteSortOrder
    """Identifier of the location associated with the cohort"""
    metadata: AirbyteSortOrder
    """Additional information or data related to the cohort"""
    name: AirbyteSortOrder
    """The name or title of the cohort"""
    owners: AirbyteSortOrder
    """The owners or administrators of the cohort"""
    popularity: AirbyteSortOrder
    """Popularity rank or score of the cohort"""
    published: AirbyteSortOrder
    """Status indicating if the cohort data is published"""
    shortcut_ids: AirbyteSortOrder
    """Identifiers of any shortcuts associated with the cohort"""
    size: AirbyteSortOrder
    """Size or scale of the cohort data"""
    type: AirbyteSortOrder
    """The type or category of the cohort"""
    view_count: AirbyteSortOrder
    """The total count of views on the cohort data"""
    viewers: AirbyteSortOrder
    """Users or viewers who have access to the cohort data"""


# Entity-specific condition types for cohorts
class CohortsEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: CohortsSearchFilter


class CohortsNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: CohortsSearchFilter


class CohortsGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: CohortsSearchFilter


class CohortsGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: CohortsSearchFilter


class CohortsLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: CohortsSearchFilter


class CohortsLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: CohortsSearchFilter


class CohortsLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: CohortsStringFilter


class CohortsFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: CohortsStringFilter


class CohortsKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: CohortsStringFilter


class CohortsContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: CohortsAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
CohortsInCondition = TypedDict("CohortsInCondition", {"in": CohortsInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

CohortsNotCondition = TypedDict("CohortsNotCondition", {"not": "CohortsCondition"}, total=False)
"""Negates the nested condition."""

CohortsAndCondition = TypedDict("CohortsAndCondition", {"and": "list[CohortsCondition]"}, total=False)
"""True if all nested conditions are true."""

CohortsOrCondition = TypedDict("CohortsOrCondition", {"or": "list[CohortsCondition]"}, total=False)
"""True if any nested condition is true."""

CohortsAnyCondition = TypedDict("CohortsAnyCondition", {"any": CohortsAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all cohorts condition types
CohortsCondition = (
    CohortsEqCondition
    | CohortsNeqCondition
    | CohortsGtCondition
    | CohortsGteCondition
    | CohortsLtCondition
    | CohortsLteCondition
    | CohortsInCondition
    | CohortsLikeCondition
    | CohortsFuzzyCondition
    | CohortsKeywordCondition
    | CohortsContainsCondition
    | CohortsNotCondition
    | CohortsAndCondition
    | CohortsOrCondition
    | CohortsAnyCondition
)


class CohortsSearchQuery(TypedDict, total=False):
    """Search query for cohorts entity."""
    filter: CohortsCondition
    sort: list[CohortsSortFilter]


# ===== EVENTS_LIST SEARCH TYPES =====

class EventsListSearchFilter(TypedDict, total=False):
    """Available fields for filtering events_list search queries."""
    autohidden: bool | None
    """Whether the event is auto-hidden"""
    clusters_hidden: bool | None
    """Whether the event is hidden from clusters"""
    deleted: bool | None
    """Whether the event is deleted"""
    display: str | None
    """Display name of the event"""
    flow_hidden: bool | None
    """Whether the event is hidden from Pathfinder"""
    hidden: bool | None
    """Whether the event is hidden"""
    id: float | None
    """Unique identifier for the event type"""
    in_waitroom: bool | None
    """Whether the event is in the waitroom"""
    name: str | None
    """Name of the event type"""
    non_active: bool | None
    """Whether the event is marked as inactive"""
    timeline_hidden: Any
    """Whether the event is hidden from the timeline"""
    totals: float | None
    """Total number of times the event occurred this week"""
    totals_delta: float | None
    """Change in totals from the previous period"""
    value: str | None
    """Raw event name in the data"""


class EventsListInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    autohidden: list[bool]
    """Whether the event is auto-hidden"""
    clusters_hidden: list[bool]
    """Whether the event is hidden from clusters"""
    deleted: list[bool]
    """Whether the event is deleted"""
    display: list[str]
    """Display name of the event"""
    flow_hidden: list[bool]
    """Whether the event is hidden from Pathfinder"""
    hidden: list[bool]
    """Whether the event is hidden"""
    id: list[float]
    """Unique identifier for the event type"""
    in_waitroom: list[bool]
    """Whether the event is in the waitroom"""
    name: list[str]
    """Name of the event type"""
    non_active: list[bool]
    """Whether the event is marked as inactive"""
    timeline_hidden: list[Any]
    """Whether the event is hidden from the timeline"""
    totals: list[float]
    """Total number of times the event occurred this week"""
    totals_delta: list[float]
    """Change in totals from the previous period"""
    value: list[str]
    """Raw event name in the data"""


class EventsListAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    autohidden: Any
    """Whether the event is auto-hidden"""
    clusters_hidden: Any
    """Whether the event is hidden from clusters"""
    deleted: Any
    """Whether the event is deleted"""
    display: Any
    """Display name of the event"""
    flow_hidden: Any
    """Whether the event is hidden from Pathfinder"""
    hidden: Any
    """Whether the event is hidden"""
    id: Any
    """Unique identifier for the event type"""
    in_waitroom: Any
    """Whether the event is in the waitroom"""
    name: Any
    """Name of the event type"""
    non_active: Any
    """Whether the event is marked as inactive"""
    timeline_hidden: Any
    """Whether the event is hidden from the timeline"""
    totals: Any
    """Total number of times the event occurred this week"""
    totals_delta: Any
    """Change in totals from the previous period"""
    value: Any
    """Raw event name in the data"""


class EventsListStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    autohidden: str
    """Whether the event is auto-hidden"""
    clusters_hidden: str
    """Whether the event is hidden from clusters"""
    deleted: str
    """Whether the event is deleted"""
    display: str
    """Display name of the event"""
    flow_hidden: str
    """Whether the event is hidden from Pathfinder"""
    hidden: str
    """Whether the event is hidden"""
    id: str
    """Unique identifier for the event type"""
    in_waitroom: str
    """Whether the event is in the waitroom"""
    name: str
    """Name of the event type"""
    non_active: str
    """Whether the event is marked as inactive"""
    timeline_hidden: str
    """Whether the event is hidden from the timeline"""
    totals: str
    """Total number of times the event occurred this week"""
    totals_delta: str
    """Change in totals from the previous period"""
    value: str
    """Raw event name in the data"""


class EventsListSortFilter(TypedDict, total=False):
    """Available fields for sorting events_list search results."""
    autohidden: AirbyteSortOrder
    """Whether the event is auto-hidden"""
    clusters_hidden: AirbyteSortOrder
    """Whether the event is hidden from clusters"""
    deleted: AirbyteSortOrder
    """Whether the event is deleted"""
    display: AirbyteSortOrder
    """Display name of the event"""
    flow_hidden: AirbyteSortOrder
    """Whether the event is hidden from Pathfinder"""
    hidden: AirbyteSortOrder
    """Whether the event is hidden"""
    id: AirbyteSortOrder
    """Unique identifier for the event type"""
    in_waitroom: AirbyteSortOrder
    """Whether the event is in the waitroom"""
    name: AirbyteSortOrder
    """Name of the event type"""
    non_active: AirbyteSortOrder
    """Whether the event is marked as inactive"""
    timeline_hidden: AirbyteSortOrder
    """Whether the event is hidden from the timeline"""
    totals: AirbyteSortOrder
    """Total number of times the event occurred this week"""
    totals_delta: AirbyteSortOrder
    """Change in totals from the previous period"""
    value: AirbyteSortOrder
    """Raw event name in the data"""


# Entity-specific condition types for events_list
class EventsListEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: EventsListSearchFilter


class EventsListNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: EventsListSearchFilter


class EventsListGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: EventsListSearchFilter


class EventsListGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: EventsListSearchFilter


class EventsListLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: EventsListSearchFilter


class EventsListLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: EventsListSearchFilter


class EventsListLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: EventsListStringFilter


class EventsListFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: EventsListStringFilter


class EventsListKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: EventsListStringFilter


class EventsListContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: EventsListAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
EventsListInCondition = TypedDict("EventsListInCondition", {"in": EventsListInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

EventsListNotCondition = TypedDict("EventsListNotCondition", {"not": "EventsListCondition"}, total=False)
"""Negates the nested condition."""

EventsListAndCondition = TypedDict("EventsListAndCondition", {"and": "list[EventsListCondition]"}, total=False)
"""True if all nested conditions are true."""

EventsListOrCondition = TypedDict("EventsListOrCondition", {"or": "list[EventsListCondition]"}, total=False)
"""True if any nested condition is true."""

EventsListAnyCondition = TypedDict("EventsListAnyCondition", {"any": EventsListAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all events_list condition types
EventsListCondition = (
    EventsListEqCondition
    | EventsListNeqCondition
    | EventsListGtCondition
    | EventsListGteCondition
    | EventsListLtCondition
    | EventsListLteCondition
    | EventsListInCondition
    | EventsListLikeCondition
    | EventsListFuzzyCondition
    | EventsListKeywordCondition
    | EventsListContainsCondition
    | EventsListNotCondition
    | EventsListAndCondition
    | EventsListOrCondition
    | EventsListAnyCondition
)


class EventsListSearchQuery(TypedDict, total=False):
    """Search query for events_list entity."""
    filter: EventsListCondition
    sort: list[EventsListSortFilter]


# ===== ACTIVE_USERS SEARCH TYPES =====

class ActiveUsersSearchFilter(TypedDict, total=False):
    """Available fields for filtering active_users search queries."""
    date: str | None
    """The date for which the active user data is reported"""
    statistics: dict[str, Any] | None
    """The statistics related to the active users for the given date"""


class ActiveUsersInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    date: list[str]
    """The date for which the active user data is reported"""
    statistics: list[dict[str, Any]]
    """The statistics related to the active users for the given date"""


class ActiveUsersAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    date: Any
    """The date for which the active user data is reported"""
    statistics: Any
    """The statistics related to the active users for the given date"""


class ActiveUsersStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    date: str
    """The date for which the active user data is reported"""
    statistics: str
    """The statistics related to the active users for the given date"""


class ActiveUsersSortFilter(TypedDict, total=False):
    """Available fields for sorting active_users search results."""
    date: AirbyteSortOrder
    """The date for which the active user data is reported"""
    statistics: AirbyteSortOrder
    """The statistics related to the active users for the given date"""


# Entity-specific condition types for active_users
class ActiveUsersEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: ActiveUsersSearchFilter


class ActiveUsersNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: ActiveUsersSearchFilter


class ActiveUsersGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: ActiveUsersSearchFilter


class ActiveUsersGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: ActiveUsersSearchFilter


class ActiveUsersLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: ActiveUsersSearchFilter


class ActiveUsersLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: ActiveUsersSearchFilter


class ActiveUsersLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: ActiveUsersStringFilter


class ActiveUsersFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: ActiveUsersStringFilter


class ActiveUsersKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: ActiveUsersStringFilter


class ActiveUsersContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: ActiveUsersAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
ActiveUsersInCondition = TypedDict("ActiveUsersInCondition", {"in": ActiveUsersInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

ActiveUsersNotCondition = TypedDict("ActiveUsersNotCondition", {"not": "ActiveUsersCondition"}, total=False)
"""Negates the nested condition."""

ActiveUsersAndCondition = TypedDict("ActiveUsersAndCondition", {"and": "list[ActiveUsersCondition]"}, total=False)
"""True if all nested conditions are true."""

ActiveUsersOrCondition = TypedDict("ActiveUsersOrCondition", {"or": "list[ActiveUsersCondition]"}, total=False)
"""True if any nested condition is true."""

ActiveUsersAnyCondition = TypedDict("ActiveUsersAnyCondition", {"any": ActiveUsersAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all active_users condition types
ActiveUsersCondition = (
    ActiveUsersEqCondition
    | ActiveUsersNeqCondition
    | ActiveUsersGtCondition
    | ActiveUsersGteCondition
    | ActiveUsersLtCondition
    | ActiveUsersLteCondition
    | ActiveUsersInCondition
    | ActiveUsersLikeCondition
    | ActiveUsersFuzzyCondition
    | ActiveUsersKeywordCondition
    | ActiveUsersContainsCondition
    | ActiveUsersNotCondition
    | ActiveUsersAndCondition
    | ActiveUsersOrCondition
    | ActiveUsersAnyCondition
)


class ActiveUsersSearchQuery(TypedDict, total=False):
    """Search query for active_users entity."""
    filter: ActiveUsersCondition
    sort: list[ActiveUsersSortFilter]


# ===== AVERAGE_SESSION_LENGTH SEARCH TYPES =====

class AverageSessionLengthSearchFilter(TypedDict, total=False):
    """Available fields for filtering average_session_length search queries."""
    date: str | None
    """The date on which the session occurred"""
    length: float | None
    """The duration of the session in seconds"""


class AverageSessionLengthInFilter(TypedDict, total=False):
    """Available fields for 'in' condition (values are lists)."""
    date: list[str]
    """The date on which the session occurred"""
    length: list[float]
    """The duration of the session in seconds"""


class AverageSessionLengthAnyValueFilter(TypedDict, total=False):
    """Available fields with Any value type. Used for 'contains' and 'any' conditions."""
    date: Any
    """The date on which the session occurred"""
    length: Any
    """The duration of the session in seconds"""


class AverageSessionLengthStringFilter(TypedDict, total=False):
    """String fields for text search conditions (like, fuzzy, keyword)."""
    date: str
    """The date on which the session occurred"""
    length: str
    """The duration of the session in seconds"""


class AverageSessionLengthSortFilter(TypedDict, total=False):
    """Available fields for sorting average_session_length search results."""
    date: AirbyteSortOrder
    """The date on which the session occurred"""
    length: AirbyteSortOrder
    """The duration of the session in seconds"""


# Entity-specific condition types for average_session_length
class AverageSessionLengthEqCondition(TypedDict, total=False):
    """Equal to: field equals value."""
    eq: AverageSessionLengthSearchFilter


class AverageSessionLengthNeqCondition(TypedDict, total=False):
    """Not equal to: field does not equal value."""
    neq: AverageSessionLengthSearchFilter


class AverageSessionLengthGtCondition(TypedDict, total=False):
    """Greater than: field > value."""
    gt: AverageSessionLengthSearchFilter


class AverageSessionLengthGteCondition(TypedDict, total=False):
    """Greater than or equal: field >= value."""
    gte: AverageSessionLengthSearchFilter


class AverageSessionLengthLtCondition(TypedDict, total=False):
    """Less than: field < value."""
    lt: AverageSessionLengthSearchFilter


class AverageSessionLengthLteCondition(TypedDict, total=False):
    """Less than or equal: field <= value."""
    lte: AverageSessionLengthSearchFilter


class AverageSessionLengthLikeCondition(TypedDict, total=False):
    """Partial string match with % wildcards."""
    like: AverageSessionLengthStringFilter


class AverageSessionLengthFuzzyCondition(TypedDict, total=False):
    """Ordered word text match (case-insensitive)."""
    fuzzy: AverageSessionLengthStringFilter


class AverageSessionLengthKeywordCondition(TypedDict, total=False):
    """Keyword text match (any word present)."""
    keyword: AverageSessionLengthStringFilter


class AverageSessionLengthContainsCondition(TypedDict, total=False):
    """Check if value exists in array field. Example: {"contains": {"tags": "premium"}}"""
    contains: AverageSessionLengthAnyValueFilter


# Reserved keyword conditions using functional TypedDict syntax
AverageSessionLengthInCondition = TypedDict("AverageSessionLengthInCondition", {"in": AverageSessionLengthInFilter}, total=False)
"""In list: field value is in list. Example: {"in": {"status": ["active", "pending"]}}"""

AverageSessionLengthNotCondition = TypedDict("AverageSessionLengthNotCondition", {"not": "AverageSessionLengthCondition"}, total=False)
"""Negates the nested condition."""

AverageSessionLengthAndCondition = TypedDict("AverageSessionLengthAndCondition", {"and": "list[AverageSessionLengthCondition]"}, total=False)
"""True if all nested conditions are true."""

AverageSessionLengthOrCondition = TypedDict("AverageSessionLengthOrCondition", {"or": "list[AverageSessionLengthCondition]"}, total=False)
"""True if any nested condition is true."""

AverageSessionLengthAnyCondition = TypedDict("AverageSessionLengthAnyCondition", {"any": AverageSessionLengthAnyValueFilter}, total=False)
"""Match if ANY element in array field matches nested condition. Example: {"any": {"addresses": {"eq": {"state": "CA"}}}}"""

# Union of all average_session_length condition types
AverageSessionLengthCondition = (
    AverageSessionLengthEqCondition
    | AverageSessionLengthNeqCondition
    | AverageSessionLengthGtCondition
    | AverageSessionLengthGteCondition
    | AverageSessionLengthLtCondition
    | AverageSessionLengthLteCondition
    | AverageSessionLengthInCondition
    | AverageSessionLengthLikeCondition
    | AverageSessionLengthFuzzyCondition
    | AverageSessionLengthKeywordCondition
    | AverageSessionLengthContainsCondition
    | AverageSessionLengthNotCondition
    | AverageSessionLengthAndCondition
    | AverageSessionLengthOrCondition
    | AverageSessionLengthAnyCondition
)


class AverageSessionLengthSearchQuery(TypedDict, total=False):
    """Search query for average_session_length entity."""
    filter: AverageSessionLengthCondition
    sort: list[AverageSessionLengthSortFilter]



# ===== SEARCH PARAMS =====

class AirbyteSearchParams(TypedDict, total=False):
    """Parameters for Airbyte cache search operations (generic, use entity-specific query types for better type hints)."""
    query: dict[str, Any]
    limit: int
    cursor: str
    fields: list[list[str]]



"""
Connector model for amplitude.

This file is auto-generated from the connector definition at build time.
DO NOT EDIT MANUALLY - changes will be overwritten on next generation.
"""

from __future__ import annotations

from ._vendored.connector_sdk.types import (
    Action,
    AuthConfig,
    AuthType,
    ConnectorModel,
    EndpointDefinition,
    EntityDefinition,
)
from ._vendored.connector_sdk.schema.security import (
    AirbyteAuthConfig,
    AuthConfigFieldSpec,
)
from uuid import (
    UUID,
)

AmplitudeConnectorModel: ConnectorModel = ConnectorModel(
    id=UUID('fa9f58c6-2d03-4237-aaa4-07d75e0c1396'),
    name='amplitude',
    version='1.0.1',
    base_url='https://amplitude.com/api',
    auth=AuthConfig(
        type=AuthType.BASIC,
        user_config_spec=AirbyteAuthConfig(
            title='API Key Authentication',
            type='object',
            required=['api_key', 'secret_key'],
            properties={
                'api_key': AuthConfigFieldSpec(
                    title='API Key',
                    description='Your Amplitude project API key. Find it in Settings > Projects in your Amplitude account.\n',
                ),
                'secret_key': AuthConfigFieldSpec(
                    title='Secret Key',
                    description='Your Amplitude project secret key. Find it in Settings > Projects in your Amplitude account.\n',
                ),
            },
            auth_mapping={'username': '${api_key}', 'password': '${secret_key}'},
            replication_auth_key_mapping={'api_key': 'api_key', 'secret_key': 'secret_key'},
        ),
    ),
    entities=[
        EntityDefinition(
            name='annotations',
            stream_name='annotations',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/2/annotations',
                    action=Action.LIST,
                    description='Returns all chart annotations for the project.',
                    response_schema={
                        'type': 'object',
                        'description': 'List of annotations',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'A chart annotation object',
                                    'properties': {
                                        'id': {'type': 'integer', 'description': 'Unique identifier for the annotation'},
                                        'date': {
                                            'type': ['null', 'string'],
                                            'format': 'date',
                                            'description': 'The date of the annotation',
                                        },
                                        'details': {
                                            'type': ['null', 'string'],
                                            'description': 'Additional details or information about the annotation',
                                        },
                                        'label': {
                                            'type': ['null', 'string'],
                                            'description': 'The label or title of the annotation',
                                        },
                                    },
                                    'required': ['id'],
                                    'x-airbyte-entity-name': 'annotations',
                                    'x-airbyte-stream-name': 'annotations',
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                    preferred_for_check=True,
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/3/annotations/{annotation_id}',
                    action=Action.GET,
                    description='Retrieves a single chart annotation by ID.',
                    path_params=['annotation_id'],
                    path_params_schema={
                        'annotation_id': {'type': 'integer', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Single annotation response',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'A chart annotation object (v3 API format)',
                                'properties': {
                                    'id': {'type': 'integer', 'description': 'Unique identifier for the annotation'},
                                    'start': {
                                        'type': ['null', 'string'],
                                        'description': 'Start timestamp in ISO 8601 format',
                                    },
                                    'end': {
                                        'type': ['null', 'string'],
                                        'description': 'End timestamp in ISO 8601 format',
                                    },
                                    'label': {
                                        'type': ['null', 'string'],
                                        'description': 'The label or title of the annotation',
                                    },
                                    'details': {
                                        'type': ['null', 'string'],
                                        'description': 'Additional details about the annotation',
                                    },
                                    'category': {
                                        'type': ['null', 'object'],
                                        'description': 'The annotation category',
                                        'properties': {
                                            'id': {'type': 'integer', 'description': 'Category ID'},
                                            'category': {'type': 'string', 'description': 'Category name'},
                                        },
                                    },
                                    'chart_id': {
                                        'type': ['null', 'string'],
                                        'description': 'The chart ID this annotation is associated with',
                                    },
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'A chart annotation object',
                'properties': {
                    'id': {'type': 'integer', 'description': 'Unique identifier for the annotation'},
                    'date': {
                        'type': ['null', 'string'],
                        'format': 'date',
                        'description': 'The date of the annotation',
                    },
                    'details': {
                        'type': ['null', 'string'],
                        'description': 'Additional details or information about the annotation',
                    },
                    'label': {
                        'type': ['null', 'string'],
                        'description': 'The label or title of the annotation',
                    },
                },
                'required': ['id'],
                'x-airbyte-entity-name': 'annotations',
                'x-airbyte-stream-name': 'annotations',
            },
        ),
        EntityDefinition(
            name='cohorts',
            stream_name='cohorts',
            actions=[Action.LIST, Action.GET],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/3/cohorts',
                    action=Action.LIST,
                    description='Returns all cohorts for the project.',
                    response_schema={
                        'type': 'object',
                        'description': 'List of cohorts',
                        'properties': {
                            'cohorts': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'A user cohort object',
                                    'properties': {
                                        'appId': {
                                            'type': ['null', 'integer'],
                                            'description': 'The unique identifier of the application',
                                        },
                                        'archived': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the cohort is archived',
                                        },
                                        'chart_id': {
                                            'type': ['null', 'string'],
                                            'description': 'The chart ID associated with the cohort',
                                        },
                                        'createdAt': {
                                            'type': ['null', 'integer'],
                                            'description': 'Timestamp when the cohort was created',
                                        },
                                        'definition': {
                                            'type': ['null', 'object'],
                                            'description': 'The definition or criteria for the cohort',
                                        },
                                        'description': {
                                            'type': ['null', 'string'],
                                            'description': 'A description of the cohort',
                                        },
                                        'edit_id': {
                                            'type': ['null', 'string'],
                                            'description': 'The edit ID for version control',
                                        },
                                        'finished': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the cohort computation has finished',
                                        },
                                        'hidden': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the cohort is hidden from view',
                                        },
                                        'id': {
                                            'type': ['null', 'string'],
                                            'description': 'Unique identifier for the cohort',
                                        },
                                        'is_official_content': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the cohort is official content',
                                        },
                                        'is_predictive': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the cohort is predictive',
                                        },
                                        'lastComputed': {
                                            'type': ['null', 'integer'],
                                            'description': 'Timestamp of the last computation',
                                        },
                                        'lastMod': {
                                            'type': ['null', 'integer'],
                                            'description': 'Timestamp of the last modification',
                                        },
                                        'last_viewed': {
                                            'type': ['null', 'integer'],
                                            'description': 'Timestamp when the cohort was last viewed',
                                        },
                                        'location_id': {
                                            'type': ['null', 'string'],
                                            'description': 'Location identifier',
                                        },
                                        'metadata': {
                                            'type': ['null', 'array'],
                                            'description': 'Additional metadata',
                                            'items': {'type': 'string'},
                                        },
                                        'name': {
                                            'type': ['null', 'string'],
                                            'description': 'The name of the cohort',
                                        },
                                        'owners': {
                                            'type': ['null', 'array'],
                                            'description': 'The owners of the cohort',
                                            'items': {'type': 'string'},
                                        },
                                        'popularity': {
                                            'type': ['null', 'integer'],
                                            'description': 'Popularity score of the cohort',
                                        },
                                        'published': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the cohort is published',
                                        },
                                        'shortcut_ids': {
                                            'type': ['null', 'array'],
                                            'description': 'Shortcut identifiers',
                                            'items': {'type': 'string'},
                                        },
                                        'size': {
                                            'type': ['null', 'integer'],
                                            'description': 'Number of users in the cohort',
                                        },
                                        'type': {
                                            'type': ['null', 'string'],
                                            'description': 'The type of cohort',
                                        },
                                        'view_count': {
                                            'type': ['null', 'integer'],
                                            'description': 'Number of views',
                                        },
                                        'viewers': {
                                            'type': ['null', 'array'],
                                            'description': 'Users who have viewed the cohort',
                                            'items': {'type': 'string'},
                                        },
                                        'include_data_app_types': {
                                            'type': ['null', 'array'],
                                            'description': 'Data app types to include',
                                            'items': {'type': 'string'},
                                        },
                                        'per_app_metadata': {
                                            'type': ['null', 'object'],
                                            'description': 'Per-application metadata',
                                        },
                                        'cohort_definition_type': {
                                            'type': ['null', 'string'],
                                            'description': 'Type of cohort definition',
                                        },
                                        'cohort_output_type': {
                                            'type': ['null', 'string'],
                                            'description': 'Output type for the cohort',
                                        },
                                        'is_generated_content': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the cohort is generated content',
                                        },
                                    },
                                    'required': ['id'],
                                    'x-airbyte-entity-name': 'cohorts',
                                    'x-airbyte-stream-name': 'cohorts',
                                },
                            },
                        },
                    },
                    record_extractor='$.cohorts',
                ),
                Action.GET: EndpointDefinition(
                    method='GET',
                    path='/3/cohorts/{cohort_id}',
                    action=Action.GET,
                    description='Retrieves a single cohort by ID.',
                    path_params=['cohort_id'],
                    path_params_schema={
                        'cohort_id': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Single cohort response wrapper',
                        'properties': {
                            'cohort': {
                                'type': 'object',
                                'description': 'A user cohort object',
                                'properties': {
                                    'appId': {
                                        'type': ['null', 'integer'],
                                        'description': 'The unique identifier of the application',
                                    },
                                    'archived': {
                                        'type': ['null', 'boolean'],
                                        'description': 'Whether the cohort is archived',
                                    },
                                    'chart_id': {
                                        'type': ['null', 'string'],
                                        'description': 'The chart ID associated with the cohort',
                                    },
                                    'createdAt': {
                                        'type': ['null', 'integer'],
                                        'description': 'Timestamp when the cohort was created',
                                    },
                                    'definition': {
                                        'type': ['null', 'object'],
                                        'description': 'The definition or criteria for the cohort',
                                    },
                                    'description': {
                                        'type': ['null', 'string'],
                                        'description': 'A description of the cohort',
                                    },
                                    'edit_id': {
                                        'type': ['null', 'string'],
                                        'description': 'The edit ID for version control',
                                    },
                                    'finished': {
                                        'type': ['null', 'boolean'],
                                        'description': 'Whether the cohort computation has finished',
                                    },
                                    'hidden': {
                                        'type': ['null', 'boolean'],
                                        'description': 'Whether the cohort is hidden from view',
                                    },
                                    'id': {
                                        'type': ['null', 'string'],
                                        'description': 'Unique identifier for the cohort',
                                    },
                                    'is_official_content': {
                                        'type': ['null', 'boolean'],
                                        'description': 'Whether the cohort is official content',
                                    },
                                    'is_predictive': {
                                        'type': ['null', 'boolean'],
                                        'description': 'Whether the cohort is predictive',
                                    },
                                    'lastComputed': {
                                        'type': ['null', 'integer'],
                                        'description': 'Timestamp of the last computation',
                                    },
                                    'lastMod': {
                                        'type': ['null', 'integer'],
                                        'description': 'Timestamp of the last modification',
                                    },
                                    'last_viewed': {
                                        'type': ['null', 'integer'],
                                        'description': 'Timestamp when the cohort was last viewed',
                                    },
                                    'location_id': {
                                        'type': ['null', 'string'],
                                        'description': 'Location identifier',
                                    },
                                    'metadata': {
                                        'type': ['null', 'array'],
                                        'description': 'Additional metadata',
                                        'items': {'type': 'string'},
                                    },
                                    'name': {
                                        'type': ['null', 'string'],
                                        'description': 'The name of the cohort',
                                    },
                                    'owners': {
                                        'type': ['null', 'array'],
                                        'description': 'The owners of the cohort',
                                        'items': {'type': 'string'},
                                    },
                                    'popularity': {
                                        'type': ['null', 'integer'],
                                        'description': 'Popularity score of the cohort',
                                    },
                                    'published': {
                                        'type': ['null', 'boolean'],
                                        'description': 'Whether the cohort is published',
                                    },
                                    'shortcut_ids': {
                                        'type': ['null', 'array'],
                                        'description': 'Shortcut identifiers',
                                        'items': {'type': 'string'},
                                    },
                                    'size': {
                                        'type': ['null', 'integer'],
                                        'description': 'Number of users in the cohort',
                                    },
                                    'type': {
                                        'type': ['null', 'string'],
                                        'description': 'The type of cohort',
                                    },
                                    'view_count': {
                                        'type': ['null', 'integer'],
                                        'description': 'Number of views',
                                    },
                                    'viewers': {
                                        'type': ['null', 'array'],
                                        'description': 'Users who have viewed the cohort',
                                        'items': {'type': 'string'},
                                    },
                                    'include_data_app_types': {
                                        'type': ['null', 'array'],
                                        'description': 'Data app types to include',
                                        'items': {'type': 'string'},
                                    },
                                    'per_app_metadata': {
                                        'type': ['null', 'object'],
                                        'description': 'Per-application metadata',
                                    },
                                    'cohort_definition_type': {
                                        'type': ['null', 'string'],
                                        'description': 'Type of cohort definition',
                                    },
                                    'cohort_output_type': {
                                        'type': ['null', 'string'],
                                        'description': 'Output type for the cohort',
                                    },
                                    'is_generated_content': {
                                        'type': ['null', 'boolean'],
                                        'description': 'Whether the cohort is generated content',
                                    },
                                },
                                'required': ['id'],
                                'x-airbyte-entity-name': 'cohorts',
                                'x-airbyte-stream-name': 'cohorts',
                            },
                        },
                    },
                    record_extractor='$.cohort',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'A user cohort object',
                'properties': {
                    'appId': {
                        'type': ['null', 'integer'],
                        'description': 'The unique identifier of the application',
                    },
                    'archived': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the cohort is archived',
                    },
                    'chart_id': {
                        'type': ['null', 'string'],
                        'description': 'The chart ID associated with the cohort',
                    },
                    'createdAt': {
                        'type': ['null', 'integer'],
                        'description': 'Timestamp when the cohort was created',
                    },
                    'definition': {
                        'type': ['null', 'object'],
                        'description': 'The definition or criteria for the cohort',
                    },
                    'description': {
                        'type': ['null', 'string'],
                        'description': 'A description of the cohort',
                    },
                    'edit_id': {
                        'type': ['null', 'string'],
                        'description': 'The edit ID for version control',
                    },
                    'finished': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the cohort computation has finished',
                    },
                    'hidden': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the cohort is hidden from view',
                    },
                    'id': {
                        'type': ['null', 'string'],
                        'description': 'Unique identifier for the cohort',
                    },
                    'is_official_content': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the cohort is official content',
                    },
                    'is_predictive': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the cohort is predictive',
                    },
                    'lastComputed': {
                        'type': ['null', 'integer'],
                        'description': 'Timestamp of the last computation',
                    },
                    'lastMod': {
                        'type': ['null', 'integer'],
                        'description': 'Timestamp of the last modification',
                    },
                    'last_viewed': {
                        'type': ['null', 'integer'],
                        'description': 'Timestamp when the cohort was last viewed',
                    },
                    'location_id': {
                        'type': ['null', 'string'],
                        'description': 'Location identifier',
                    },
                    'metadata': {
                        'type': ['null', 'array'],
                        'description': 'Additional metadata',
                        'items': {'type': 'string'},
                    },
                    'name': {
                        'type': ['null', 'string'],
                        'description': 'The name of the cohort',
                    },
                    'owners': {
                        'type': ['null', 'array'],
                        'description': 'The owners of the cohort',
                        'items': {'type': 'string'},
                    },
                    'popularity': {
                        'type': ['null', 'integer'],
                        'description': 'Popularity score of the cohort',
                    },
                    'published': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the cohort is published',
                    },
                    'shortcut_ids': {
                        'type': ['null', 'array'],
                        'description': 'Shortcut identifiers',
                        'items': {'type': 'string'},
                    },
                    'size': {
                        'type': ['null', 'integer'],
                        'description': 'Number of users in the cohort',
                    },
                    'type': {
                        'type': ['null', 'string'],
                        'description': 'The type of cohort',
                    },
                    'view_count': {
                        'type': ['null', 'integer'],
                        'description': 'Number of views',
                    },
                    'viewers': {
                        'type': ['null', 'array'],
                        'description': 'Users who have viewed the cohort',
                        'items': {'type': 'string'},
                    },
                    'include_data_app_types': {
                        'type': ['null', 'array'],
                        'description': 'Data app types to include',
                        'items': {'type': 'string'},
                    },
                    'per_app_metadata': {
                        'type': ['null', 'object'],
                        'description': 'Per-application metadata',
                    },
                    'cohort_definition_type': {
                        'type': ['null', 'string'],
                        'description': 'Type of cohort definition',
                    },
                    'cohort_output_type': {
                        'type': ['null', 'string'],
                        'description': 'Output type for the cohort',
                    },
                    'is_generated_content': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the cohort is generated content',
                    },
                },
                'required': ['id'],
                'x-airbyte-entity-name': 'cohorts',
                'x-airbyte-stream-name': 'cohorts',
            },
        ),
        EntityDefinition(
            name='events_list',
            stream_name='events_list',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/2/events/list',
                    action=Action.LIST,
                    description="Returns the list of event types with the current week's totals, unique users, and percentage of DAU.\n",
                    response_schema={
                        'type': 'object',
                        'description': 'List of event types',
                        'properties': {
                            'data': {
                                'type': 'array',
                                'items': {
                                    'type': 'object',
                                    'description': 'An event type definition with weekly totals',
                                    'properties': {
                                        'autohidden': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event is auto-hidden',
                                        },
                                        'clusters_hidden': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event is hidden from clusters',
                                        },
                                        'deleted': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event is deleted',
                                        },
                                        'display': {
                                            'type': ['null', 'string'],
                                            'description': 'Display name of the event',
                                        },
                                        'flow_hidden': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event is hidden from Pathfinder',
                                        },
                                        'hidden': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event is hidden',
                                        },
                                        'id': {'type': 'number', 'description': 'Unique identifier for the event type'},
                                        'in_waitroom': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event is in the waitroom',
                                        },
                                        'name': {
                                            'type': ['null', 'string'],
                                            'description': 'Name of the event type',
                                        },
                                        'non_active': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event is marked as inactive',
                                        },
                                        'timeline_hidden': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event is hidden from the timeline',
                                        },
                                        'totals': {
                                            'type': ['null', 'number'],
                                            'description': 'Total number of times the event occurred this week',
                                        },
                                        'totals_delta': {
                                            'type': ['null', 'number'],
                                            'description': 'Change in totals from the previous period',
                                        },
                                        'value': {
                                            'type': ['null', 'string'],
                                            'description': 'Raw event name in the data',
                                        },
                                        'waitroom_approved': {
                                            'type': ['null', 'boolean'],
                                            'description': 'Whether the event has been approved from the waitroom',
                                        },
                                    },
                                    'required': ['id'],
                                    'x-airbyte-entity-name': 'events_list',
                                    'x-airbyte-stream-name': 'events_list',
                                },
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'An event type definition with weekly totals',
                'properties': {
                    'autohidden': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event is auto-hidden',
                    },
                    'clusters_hidden': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event is hidden from clusters',
                    },
                    'deleted': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event is deleted',
                    },
                    'display': {
                        'type': ['null', 'string'],
                        'description': 'Display name of the event',
                    },
                    'flow_hidden': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event is hidden from Pathfinder',
                    },
                    'hidden': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event is hidden',
                    },
                    'id': {'type': 'number', 'description': 'Unique identifier for the event type'},
                    'in_waitroom': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event is in the waitroom',
                    },
                    'name': {
                        'type': ['null', 'string'],
                        'description': 'Name of the event type',
                    },
                    'non_active': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event is marked as inactive',
                    },
                    'timeline_hidden': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event is hidden from the timeline',
                    },
                    'totals': {
                        'type': ['null', 'number'],
                        'description': 'Total number of times the event occurred this week',
                    },
                    'totals_delta': {
                        'type': ['null', 'number'],
                        'description': 'Change in totals from the previous period',
                    },
                    'value': {
                        'type': ['null', 'string'],
                        'description': 'Raw event name in the data',
                    },
                    'waitroom_approved': {
                        'type': ['null', 'boolean'],
                        'description': 'Whether the event has been approved from the waitroom',
                    },
                },
                'required': ['id'],
                'x-airbyte-entity-name': 'events_list',
                'x-airbyte-stream-name': 'events_list',
            },
        ),
        EntityDefinition(
            name='active_users',
            stream_name='active_users',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/2/users',
                    action=Action.LIST,
                    description='Returns the number of active or new users for each day in the specified date range.\n',
                    query_params=[
                        'start',
                        'end',
                        'm',
                        'i',
                        'g',
                    ],
                    query_params_schema={
                        'start': {'type': 'string', 'required': True},
                        'end': {'type': 'string', 'required': True},
                        'm': {
                            'type': 'string',
                            'required': False,
                            'default': 'active',
                        },
                        'i': {
                            'type': 'integer',
                            'required': False,
                            'default': 1,
                        },
                        'g': {'type': 'string', 'required': False},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Active users response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Active or new user count data',
                                'properties': {
                                    'series': {
                                        'type': ['null', 'array'],
                                        'description': 'An array with one element for each group, where each element is an array of metric values per date in xValues.\n',
                                        'items': {
                                            'type': 'array',
                                            'items': {'type': 'number'},
                                        },
                                    },
                                    'seriesCollapsed': {
                                        'type': ['null', 'array'],
                                        'description': 'Collapsed series values',
                                        'items': {
                                            'type': 'array',
                                            'items': {'type': 'number'},
                                        },
                                    },
                                    'seriesLabels': {
                                        'type': ['null', 'array'],
                                        'description': 'Labels for each series group',
                                        'items': {
                                            'type': ['string', 'integer'],
                                        },
                                    },
                                    'seriesMeta': {
                                        'type': ['null', 'array'],
                                        'description': 'Metadata for each segment',
                                        'items': {
                                            'type': 'object',
                                            'properties': {
                                                'segmentIndex': {'type': 'integer'},
                                            },
                                        },
                                    },
                                    'xValues': {
                                        'type': ['null', 'array'],
                                        'description': 'Array of dates in YYYY-MM-DD format',
                                        'items': {'type': 'string'},
                                    },
                                },
                                'x-airbyte-entity-name': 'active_users',
                                'x-airbyte-stream-name': 'active_users',
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Active or new user count data',
                'properties': {
                    'series': {
                        'type': ['null', 'array'],
                        'description': 'An array with one element for each group, where each element is an array of metric values per date in xValues.\n',
                        'items': {
                            'type': 'array',
                            'items': {'type': 'number'},
                        },
                    },
                    'seriesCollapsed': {
                        'type': ['null', 'array'],
                        'description': 'Collapsed series values',
                        'items': {
                            'type': 'array',
                            'items': {'type': 'number'},
                        },
                    },
                    'seriesLabels': {
                        'type': ['null', 'array'],
                        'description': 'Labels for each series group',
                        'items': {
                            'type': ['string', 'integer'],
                        },
                    },
                    'seriesMeta': {
                        'type': ['null', 'array'],
                        'description': 'Metadata for each segment',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'segmentIndex': {'type': 'integer'},
                            },
                        },
                    },
                    'xValues': {
                        'type': ['null', 'array'],
                        'description': 'Array of dates in YYYY-MM-DD format',
                        'items': {'type': 'string'},
                    },
                },
                'x-airbyte-entity-name': 'active_users',
                'x-airbyte-stream-name': 'active_users',
            },
        ),
        EntityDefinition(
            name='average_session_length',
            stream_name='average_session_length',
            actions=[Action.LIST],
            endpoints={
                Action.LIST: EndpointDefinition(
                    method='GET',
                    path='/2/sessions/average',
                    action=Action.LIST,
                    description='Returns the average session length (in seconds) for each day in the specified date range.\n',
                    query_params=['start', 'end'],
                    query_params_schema={
                        'start': {'type': 'string', 'required': True},
                        'end': {'type': 'string', 'required': True},
                    },
                    response_schema={
                        'type': 'object',
                        'description': 'Average session length response wrapper',
                        'properties': {
                            'data': {
                                'type': 'object',
                                'description': 'Average session length data',
                                'properties': {
                                    'series': {
                                        'type': ['null', 'array'],
                                        'description': 'An array with one element which is an array of average session lengths (in seconds) for each day.\n',
                                        'items': {
                                            'type': 'array',
                                            'items': {'type': 'number'},
                                        },
                                    },
                                    'seriesCollapsed': {
                                        'type': ['null', 'array'],
                                        'description': 'Collapsed series values',
                                        'items': {
                                            'type': 'array',
                                            'items': {
                                                'type': 'object',
                                                'properties': {
                                                    'setId': {'type': 'string'},
                                                    'value': {'type': 'number'},
                                                },
                                            },
                                        },
                                    },
                                    'seriesMeta': {
                                        'type': ['null', 'array'],
                                        'description': 'Labels for each segment',
                                        'items': {
                                            'type': 'object',
                                            'properties': {
                                                'segmentIndex': {'type': 'integer'},
                                                'sessionIndex': {'type': 'integer'},
                                            },
                                        },
                                    },
                                    'xValues': {
                                        'type': ['null', 'array'],
                                        'description': 'Array of dates in YYYY-MM-DD format',
                                        'items': {'type': 'string'},
                                    },
                                },
                                'x-airbyte-entity-name': 'average_session_length',
                                'x-airbyte-stream-name': 'average_session_length',
                            },
                        },
                    },
                    record_extractor='$.data',
                ),
            },
            entity_schema={
                'type': 'object',
                'description': 'Average session length data',
                'properties': {
                    'series': {
                        'type': ['null', 'array'],
                        'description': 'An array with one element which is an array of average session lengths (in seconds) for each day.\n',
                        'items': {
                            'type': 'array',
                            'items': {'type': 'number'},
                        },
                    },
                    'seriesCollapsed': {
                        'type': ['null', 'array'],
                        'description': 'Collapsed series values',
                        'items': {
                            'type': 'array',
                            'items': {
                                'type': 'object',
                                'properties': {
                                    'setId': {'type': 'string'},
                                    'value': {'type': 'number'},
                                },
                            },
                        },
                    },
                    'seriesMeta': {
                        'type': ['null', 'array'],
                        'description': 'Labels for each segment',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'segmentIndex': {'type': 'integer'},
                                'sessionIndex': {'type': 'integer'},
                            },
                        },
                    },
                    'xValues': {
                        'type': ['null', 'array'],
                        'description': 'Array of dates in YYYY-MM-DD format',
                        'items': {'type': 'string'},
                    },
                },
                'x-airbyte-entity-name': 'average_session_length',
                'x-airbyte-stream-name': 'average_session_length',
            },
        ),
    ],
    search_field_paths={
        'annotations': [
            'date',
            'details',
            'id',
            'label',
        ],
        'cohorts': [
            'appId',
            'archived',
            'chart_id',
            'createdAt',
            'definition',
            'description',
            'edit_id',
            'finished',
            'hidden',
            'id',
            'is_official_content',
            'is_predictive',
            'lastComputed',
            'lastMod',
            'last_viewed',
            'location_id',
            'metadata',
            'metadata[]',
            'name',
            'owners',
            'owners[]',
            'popularity',
            'published',
            'shortcut_ids',
            'shortcut_ids[]',
            'size',
            'type',
            'view_count',
            'viewers',
            'viewers[]',
        ],
        'events_list': [
            'autohidden',
            'clusters_hidden',
            'deleted',
            'display',
            'flow_hidden',
            'hidden',
            'id',
            'in_waitroom',
            'name',
            'non_active',
            'timeline_hidden',
            'totals',
            'totals_delta',
            'value',
        ],
        'active_users': ['date', 'statistics'],
        'events': [
            '$insert_id',
            'adid',
            'amplitude_attribution_ids',
            'amplitude_event_type',
            'amplitude_id',
            'app',
            'city',
            'client_event_time',
            'client_upload_time',
            'country',
            'data',
            'data_type',
            'device_brand',
            'device_carrier',
            'device_family',
            'device_id',
            'device_manufacturer',
            'device_model',
            'device_type',
            'dma',
            'event_id',
            'event_properties',
            'event_time',
            'event_type',
            'global_user_properties',
            'group_properties',
            'groups',
            'idfa',
            'ip_address',
            'is_attribution_event',
            'language',
            'library',
            'location_lat',
            'location_lng',
            'os_name',
            'os_version',
            'partner_id',
            'paying',
            'plan',
            'plan.branch',
            'plan.source',
            'plan.version',
            'platform',
            'processed_time',
            'region',
            'sample_rate',
            'server_received_time',
            'server_upload_time',
            'session_id',
            'source_id',
            'start_version',
            'user_creation_time',
            'user_id',
            'user_properties',
            'uuid',
            'version_name',
        ],
        'average_session_length': ['date', 'length'],
    },
)


"""HTTPX adapter implementing the HTTP client protocol."""

from typing import Any

import httpx

from ..config import ClientConfig, ConnectionLimits, TimeoutConfig
from ..exceptions import (
    AuthenticationError,
    HTTPStatusError,
    NetworkError,
    RateLimitError,
    TimeoutError,
)
from ..protocols import HTTPResponseProtocol
from ..response import HTTPResponse


class HTTPXClient:
    """HTTPX-based implementation of the HTTP client protocol.

    This adapter wraps httpx.AsyncClient and provides the HTTPClientProtocol interface,
    allowing httpx to be swapped out for a different HTTP client in the future.
    """

    def __init__(self, config: ClientConfig | None = None) -> None:
        """Initialize the HTTPX client adapter.

        Args:
            config: Client configuration. If None, uses default configuration.
        """
        self.config = config or ClientConfig()
        self._client: httpx.AsyncClient | None = None

    def _create_client(self) -> httpx.AsyncClient:
        """Create and configure the httpx AsyncClient.

        Returns:
            Configured httpx.AsyncClient instance.
        """
        # Convert SDK config to httpx config
        limits = self._convert_limits(self.config.limits)  # type: ignore
        timeout = self._convert_timeout(self.config.timeout)  # type: ignore

        return httpx.AsyncClient(
            base_url=self.config.base_url or "",
            timeout=timeout,
            limits=limits,
            follow_redirects=self.config.follow_redirects,
        )

    def _convert_limits(self, limits: ConnectionLimits) -> httpx.Limits:
        """Convert SDK ConnectionLimits to httpx.Limits.

        Args:
            limits: SDK connection limits configuration

        Returns:
            httpx.Limits instance
        """
        return httpx.Limits(
            max_connections=limits.max_connections,
            max_keepalive_connections=limits.max_keepalive_connections,
        )

    def _convert_timeout(self, timeout: TimeoutConfig) -> httpx.Timeout:
        """Convert SDK TimeoutConfig to httpx.Timeout.

        Args:
            timeout: SDK timeout configuration

        Returns:
            httpx.Timeout instance
        """
        return httpx.Timeout(
            connect=timeout.connect,
            read=timeout.read,
            write=timeout.write,
            pool=timeout.pool,
        )

    def _convert_response(self, httpx_response: httpx.Response, *, stream: bool = False) -> HTTPResponse:
        """Convert httpx.Response to SDK HTTPResponse.

        Args:
            httpx_response: The httpx response object
            stream: Whether the response should be treated as streaming (do not eagerly read body)

        Returns:
            HTTPResponse wrapping the httpx response
        """
        return HTTPResponse(
            status_code=httpx_response.status_code,
            headers=dict(httpx_response.headers),
            # When streaming, avoid eagerly reading the body
            content=b"" if stream else httpx_response.content,
            _original_response=httpx_response,
        )

    async def request(
        self,
        method: str,
        url: str,
        *,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        data: dict[str, Any] | str | None = None,
        headers: dict[str, str] | None = None,
        **kwargs: Any,
    ) -> HTTPResponseProtocol:
        """Execute an HTTP request using httpx.

        Args:
            method: HTTP method (GET, POST, PUT, DELETE, etc.)
            url: The URL to request
            params: Query parameters to append to the URL
            json: JSON data to send in the request body
            data: Form data or raw string to send in the request body
            headers: HTTP headers to include in the request
            **kwargs: Additional httpx-specific parameters

        Returns:
            HTTPResponse with the response data.

        Raises:
            HTTPStatusError: For 4xx or 5xx HTTP status codes
            AuthenticationError: For 401 or 403 status codes
            RateLimitError: For 429 status codes
            TimeoutError: For timeout errors
            NetworkError: For network/connection errors
        """
        if self._client is None:
            self._client = self._create_client()

        # Extract stream parameter (not supported by httpx.request directly)
        stream = kwargs.pop("stream", False)

        try:
            # Execute the request
            httpx_response = await self._client.request(
                method=method,
                url=url,
                params=params,
                json=json,
                data=data,
                headers=headers,
                **kwargs,
            )

            # Convert to SDK response
            response = self._convert_response(httpx_response, stream=stream)

            # Check for HTTP errors and wrap them
            if httpx_response.status_code >= 400:
                await self._handle_http_error(httpx_response, response)

            return response

        except httpx.TimeoutException as e:
            raise TimeoutError(
                message=f"Request timed out: {e}",
                timeout_type=None,  # httpx doesn't provide specific timeout type
                original_error=e,
            ) from e

        except (httpx.ConnectError, httpx.NetworkError) as e:
            raise NetworkError(
                message=f"Network error: {e}",
                original_error=e,
            ) from e

    async def _handle_http_error(self, httpx_response: httpx.Response, sdk_response: HTTPResponse) -> None:
        """Handle HTTP error responses by raising appropriate SDK exceptions.

        Args:
            httpx_response: The original httpx response
            sdk_response: The converted SDK response

        Raises:
            AuthenticationError: For 401 or 403 status codes
            RateLimitError: For 429 status codes
            HTTPStatusError: For other 4xx or 5xx status codes
        """
        status_code = httpx_response.status_code

        # Try to get error message from response
        try:
            error_data = httpx_response.json()
            errors_list = error_data.get("errors")
            if isinstance(errors_list, list):

                def _extract_error(e: object) -> str:
                    if isinstance(e, dict):
                        return str(e.get("userPresentableMessage") or e.get("message") or e.get("error") or e)
                    return str(e)

                errors_list = ", ".join(_extract_error(e) for e in errors_list)
            error_message = errors_list or error_data.get("message") or error_data.get("error") or str(error_data)
        except Exception:
            error_message = httpx_response.text or f"HTTP {status_code} error"

        # Raise specific exceptions based on status code
        if status_code in (401, 403):
            raise AuthenticationError(
                message=f"Authentication failed: {error_message} (HTTP {status_code})",
                status_code=status_code,
                response=sdk_response,
            )

        if status_code == 429:
            # Try to parse Retry-After header
            retry_after = None
            if "retry-after" in httpx_response.headers:
                try:
                    retry_after = int(httpx_response.headers["retry-after"])
                except ValueError:
                    pass

            error_msg = f"Rate limit exceeded: {error_message}"
            if retry_after:
                error_msg += f" (retry after {retry_after}s)"

            raise RateLimitError(
                message=error_msg,
                retry_after=retry_after,
                response=sdk_response,
            )

        # Generic HTTP error
        raise HTTPStatusError(
            status_code=status_code,
            message=error_message,
            response=sdk_response,
        )

    async def aclose(self) -> None:
        """Close the HTTP client and cleanup resources."""
        if self._client is not None:
            await self._client.aclose()
            self._client = None

    async def __aenter__(self) -> "HTTPXClient":
        """Enter async context manager.

        Returns:
            The client instance.
        """
        if self._client is None:
            self._client = self._create_client()
        return self

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """Exit async context manager and cleanup resources.

        Args:
            exc_type: Exception type if an exception occurred
            exc_val: Exception value if an exception occurred
            exc_tb: Exception traceback if an exception occurred
        """
        await self.aclose()
